
# q-expansion of newform 7168.2.a.e, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7168
weight = 2
poly_data = [-2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, 1], [0, 0], [-1, 0], [0, -2], [0, 2], [6, 0], [0, 3], [0, 0], [0, -1], [10, 0], [0, -3], [-10, 0], [0, 6], [-2, 0], [0, 9], [0, 1], [0, 0], [0, -8], [12, 0], [2, 0], [-4, 0], [0, 5], [-2, 0], [-2, 0], [0, 4], [-6, 0], [0, 0], [0, 3], [12, 0], [12, 0], [0, -7], [12, 0], [0, -3], [0, 5], [20, 0], [0, -10], [0, 2], [22, 0], [0, 10], [0, 16], [0, -6], [20, 0], [2, 0], [0, 13], [22, 0], [0, 20], [16, 0], [0, 1], [0, 18], [10, 0], [0, 0], [-2, 0], [0, -11], [-6, 0], [-12, 0], [0, -14], [24, 0], [0, -17], [-10, 0], [0, -15], [0, -4], [0, -23], [8, 0], [30, 0], [0, -17], [0, -6], [-16, 0], [0, 22], [0, -6], [10, 0], [12, 0], [-8, 0], [0, 3], [0, -2], [6, 0], [0, -19], [0, -10], [-10, 0], [-6, 0], [0, -3], [0, -17], [16, 0], [26, 0], [8, 0], [0, -20], [-32, 0], [42, 0], [0, 14], [-40, 0], [0, 5], [-14, 0], [-8, 0], [0, 0], [0, -12], [16, 0], [0, -26], [30, 0], [0, -11], [0, -1], [0, 22], [0, -1], [0, -15], [-18, 0], [0, -6], [2, 0], [0, -21], [6, 0], [40, 0], [30, 0], [-8, 0], [0, -19], [-40, 0], [0, 27], [20, 0], [2, 0], [0, 17], [38, 0], [0, -19], [0, -14], [0, 28], [-36, 0], [0, -2], [0, 16], [0, -15], [0, 21], [0, -27], [26, 0], [-30, 0], [0, -24], [0, 22], [44, 0], [-32, 0], [0, -37], [42, 0], [-38, 0], [0, 12], [0, 13], [0, -22], [22, 0], [0, 7], [0, 31], [8, 0], [0, 20], [0, -32], [-30, 0], [0, -10], [-22, 0], [0, -33], [4, 0], [0, -1], [-26, 0], [0, -28], [-38, 0], [0, 16], [-12, 0], [32, 0], [6, 0], [-30, 0], [0, -28], [0, -38], [-16, 0], [-24, 0], [0, -15], [20, 0], [-6, 0], [-48, 0], [0, 20], [18, 0], [0, -38], [0, 28], [0, 18], [60, 0], [10, 0], [-6, 0], [-14, 0], [0, 20], [0, -5], [-64, 0], [0, 28], [-32, 0], [0, -29], [0, 7], [-42, 0], [36, 0], [0, 38], [0, 29], [0, 11], [28, 0], [6, 0], [34, 0], [0, 4], [0, 12], [0, -2], [0, -28], [58, 0], [-32, 0], [0, 33], [-2, 0], [16, 0], [0, 35], [-50, 0], [0, -36], [38, 0], [0, 37], [0, -4], [54, 0], [0, -6], [-20, 0], [0, -21], [34, 0], [0, -4], [36, 0], [0, 41], [-48, 0], [-62, 0], [-56, 0], [66, 0], [-16, 0], [0, 3], [0, 1], [0, 0], [-48, 0], [-8, 0], [0, 3], [0, -15], [62, 0], [32, 0], [-8, 0], [0, 38], [0, -3], [0, 7], [-40, 0], [-46, 0], [0, 23], [-24, 0], [14, 0], [0, 13], [0, 28], [40, 0], [0, -38], [0, -19], [-64, 0], [0, 27], [6, 0], [-18, 0], [8, 0], [0, -29], [0, 30], [-56, 0], [0, -23], [-70, 0], [-20, 0], [-2, 0], [0, -8], [0, -4], [0, 39], [0, 43], [0, -40], [-6, 0], [56, 0], [0, -34], [0, 0], [0, 12], [30, 0], [0, -35], [0, 47], [38, 0], [0, -24], [0, -47], [0, -38], [0, -10], [66, 0], [28, 0], [34, 0], [-40, 0], [0, 18], [0, 33], [-20, 0], [0, 39], [40, 0], [16, 0], [66, 0], [0, -60], [0, 39], [-52, 0], [-16, 0], [0, -3], [-46, 0], [-30, 0], [0, -27], [0, 23], [-56, 0], [0, -59], [0, -45], [0, -2], [-8, 0], [0, -46], [0, 7], [0, -45], [-54, 0], [0, 5], [-40, 0], [0, -26], [0, -30], [-20, 0], [0, 14], [0, 12], [-24, 0], [0, 9], [48, 0], [0, 37], [-12, 0], [0, 18], [20, 0], [30, 0], [0, -31], [40, 0], [-26, 0], [-50, 0], [0, 15], [52, 0], [0, 24], [76, 0], [42, 0], [-38, 0], [0, -2], [0, -19], [-36, 0], [0, -31], [0, -9], [0, 59], [-26, 0], [0, -27], [0, -36], [0, -19], [0, 45], [26, 0], [14, 0], [-58, 0], [0, -21], [-70, 0], [0, 42], [-68, 0], [0, 37], [0, -26], [0, 18], [0, 30], [0, 0], [0, 16], [0, 1], [48, 0], [0, 27], [-66, 0], [0, -5], [66, 0], [-70, 0], [0, -13], [8, 0], [80, 0], [0, 21], [82, 0], [60, 0], [0, 32], [0, 27], [32, 0], [0, -30], [-8, 0], [-6, 0], [0, 12], [0, 5], [16, 0], [0, 11], [-32, 0], [0, -47], [0, -21], [88, 0], [70, 0], [30, 0], [-78, 0], [0, 10], [-90, 0], [-48, 0], [58, 0], [0, 3], [24, 0], [-44, 0], [0, 12], [0, -64], [-18, 0], [-80, 0], [0, 56], [0, -32], [0, -49], [12, 0], [28, 0], [50, 0], [-42, 0], [0, -44], [0, -51], [0, 60], [48, 0], [32, 0], [-14, 0], [0, 62], [-6, 0], [0, -11], [-78, 0], [0, -51], [0, 25], [-10, 0], [0, 25], [0, -22], [0, 64], [22, 0], [0, -54], [52, 0], [-54, 0], [54, 0], [96, 0], [0, -5], [0, -52], [72, 0], [0, 49], [-26, 0], [0, -52], [0, -34], [10, 0], [0, -7], [-34, 0]]
